# /etc/ax25/node.conf - AWZNode example configuration file
#
# see node.conf(5)

# Idle timeout (seconds).
#
IdleTimeout	9000

# Timeout when gatewaying (seconds).
#
ConnTimeout	9000

# Visible hostname. Will be shown at telnet login.
#
HostName	xx0xx.ampr.org

# ReConnect flag.
# 
ReConnect	on

# "Local" network.
#
LocalNet	44.0.0.0/32

# Command aliases. See node.conf(5) for the meaning of the uppercase
# letters in the name of the alias.
#
Alias		CAllbook "telnet %{2:zone.oh7rba} 1235 %1 s"
Alias		CONVers  "telnet %{2:oh2ti} 3600 \"/n %u %{1:139}\""
#Alias		DXCluster "connect dxclx"
#Alias		ITANET	 "connect ir5apr"

# Hidden ports.
#
#HiddenPorts	itanet

# External commands. See node.conf(5) for the meaning of the uppercase
# letters in the name of the extcmd.
#
# Flags:	1	Run command through pipe
#		2	Reconnected flag
#
ExtCmd		NEstat	1	nobody	/bin/netstat netstat --inet
#ExtCmd		PMS	1	root	/usr/sbin/pms pms -u %U -o XX0XX
#ExtCmd		TPM	3	nobody	/usr/bin/finger	finger tpm
#ExtCmd		Echo	1	nobody	/bin/echo echo %U %u %S %s %P %p %R %r %T %t %% %0 %{1:foobar} %{2} %3 %4 %5

# Node ID.
#
NodeId		XXXxxx:XX0XX

# Netrom port name. This port is used for outgoing netrom connects.
#
NrPort		netrom

# Logging level
#
LogLevel	3

# The escape character (CTRL-T)
#
EscapeChar      ^T

# AWZNode Prompt
#
Prompt		"XXXxxx:XX0XX Node\nCmd: !,?,CA,CONV,B,C,D,E,F,H,HO,I,L,MH,MS,N,PI,P,R,S,TA,T,U ->"

# Password Prompt
#
PassPrompt 	"SYSPassword>"
