#ifndef _NODE_USER
#define _NODE_USER

struct user
{
        int             pid;
        int             fd;
        key_t           ipc_key;
        long            logintime;
        long            cmdtime;
        unsigned char   state;
        char            call[10];
        unsigned short  ul_type;
        unsigned short  dl_type;
        char            ul_name[32];
        char            dl_name[32];
        char            ul_port[32];
        char            dl_port[32];

        char            unused[92];
};

#endif /* _NODE_USER */
