# /etc/ax25/node.perms - AWZNode examplepermissions file
#
# see node.perms(5)

# user	type	port	passwd	perms

# User oh2bns can login without password from anywhere else but 'inet'.
#
#xx0xx	inet	*	qwerty	95
#xx0xx	*	*	*	95

# OH2RBI is a bbs so it needs escape disabled.
#
#xx0xx	*	*	*	287

# Default permissions per connection type.
#
*	ax25	*	*	31
*	netrom	*	*	31
*	local	*	*	31
*	ampr	*	*	31
*	inet	*	*	0
*	host	*	*	31
