# /etc/ax25/node.routes     AWZNode example configuration file
#
# AWZNode shows links with this order.

# Direct Routing 
# route <call> <alias> <port> d 'description'
#
route 	iw5czk-8	czkbbs		czk 	d 	'BBS Sezione ARI Massarosa'
route 	ir5apr 		aprnod		itanet 	d 	'Nodo ITANET Monte Petra Pertusa (LU)'

# Routing across another node with sending a string "C <call>"
# route <call> 		<alias> 	<port> n 	'description' 	<node>
#
route 	ik5skv-8 	skvbbs		itanet 	n 	'BBS ARI Empoli' 	ir5apr 
route 	ik5sil-8 	silbbs		itanet 	n 	'BBS ARI Pontedera ' 	ir5apr

# Routing via digipeaters
# route <call> 		<alias>		<port> 	v 	'description' 	<digipeaters>
#
route 	iz5awz 		awztcp		vhf 	v 	'AWZTCP Station' 	ik5qpz
