# /etc/ax25/node.users	AWZNode example configuration file
# Mail & Shell for Sysop and users

# callsign:password:linux username:settings
# where <settings> can be mail, shell or both separated by a semicolon
xx0xx:password:user:mail,shell
